<?php

/**
 * The kuntai class file
 * @copyright   上海锟钛软件科技有限公司 2022
 * @license     MIT
 * @author      kuntai<kuntai@kuntaisoft.cn>
 * @link        http://www.kuntaisoft.cn
 * 
 */

if (!function_exists('cal_days_in_month')) {
	define('CAL_GREGORIAN', 0);
	function cal_days_in_month($calendar, $month, $year)
	{
		return date('t', mktime(0, 0, 0, $month, 1, $year));
	}
}

//创建Errcode类
function kuntai_create_new_errcode()
{
	$ret = new stdclass();
	$ret->code = 0;
	$ret->msg  = '';
	$ret->data  = null;
	return $ret;
}

/**
 * The kuntai class.
 *
 * @package   framework
 */
class kuntai
{

	/**
	 * echo message Die
	 * @param array $Errors
	 *  like $Err[][], 2D
	 */
	public static function dieError($Errors)
	{
		if (is_array($Errors)) {
			foreach ($Errors as $msgArr) {
				if (is_array($msgArr)) {
					foreach ($msgArr as $msg) {
						echo $msg . "<br>";
					}
				} else echo $msg . "<br>";
			}
		} else echo $Errors;
		die();
	}

	/**
	 *  echo message ifdao error
	 */
	public static function dieIfDaoError()
	{
		if (!dao::isError()) return;
		kuntai::dieError(dao::getError());
		die();
	}

	/**
	 * if has message,echo and Die
	 * @param array $Errors
	 *  like $Err[][], 2D
	 */
	public static function dieIfError(&$Errors)
	{
		if (empty($Errors)) return;
		kuntai::dieError($Errors);
	}

	/**
	 * Format time 0915 to 09:15
	 *  20150810 to 2015-08-10
	 * @param  string $time
	 * @access public
	 * @return string
	 */
	public static function formatTime($time)
	{
		switch (strlen($time)) {
			case 4:
				return substr($time, 0, 2) . ':' . substr($time, 2, 2);
			case 8:
				return substr($time, 0, 4) . '-' . substr($time, 4, 2) . '-' . substr($time, 6, 2);
			default:
				return $time;
		}
	}

	/**
	 * The getBeginEndTime.
	 *
	 * @access public
	 * @param  类型  $type
	 * @param  int $delta
	 * @return array{begin,end}.
	 */
	public static function getBeginEndTime($period)
	{
		if ('' == $period) {
			$type = 'thisMonth';
		}
		//$this->app->loadClass('date');
		$period      = strtolower($period);
		$inputLength = strlen($period);
		$isNumeric   = is_numeric($period);
		//此处组合为年+月 共6位数字
		if ($isNumeric && 6 == $inputLength) {
			$period      .= '0';
			$inputLength = 7;
		}
		//此处判断为最近时间,小于等于4位数字
		if ($isNumeric && $inputLength < 4) {
			$begin = date('Y-m-d', strtotime('-' . $period . ' day'));
			$end   = date('Y-m-d', strtotime('+1 day'));
		} else if ($isNumeric && 4 == $inputLength) {
			$begin = $period . '-01-01 00:00:00';
			$end   = $period . '-12-31 23:59:59';;
		} //此处判断的7为数字组成为年+月+季度
		else if ($isNumeric && 7 == $inputLength) {
			//日期截取
			$year  = substr($period, 0, 4);
			$month = substr($period, 4, 2);
			if ('00' == $month) {
				$month = '';
			}
			$season = substr($period, 6, 1);
			if ('0' == $season) {
				$season = '';
			}
			$monthArray = kuntai::getMonthBySeason($month, $season);
			if (1 == count($monthArray)) {
				$currentMonth = current($monthArray);
				$begin        = $year . '-' . $currentMonth . '-01';
				return kuntai::getMonthBeginEnd($begin);
			} else {
				$begin          = $year . '-' . current($monthArray) . '-01 00:00:00';
				$last           = $year . '-' . end($monthArray) . '-' . '01 00:00:00';
				$Array          = kuntai::getMonthBeginEnd($last);
				$Array['begin'] = $begin;
				return $Array;
			}
		} else {
			switch ($period) {
				case 'today':
					$begin = kuntai::formatTime(date::today());
					$end   = $begin;
					break;
				case 'yesterday':
					$begin = kuntai::formatTime(date::yesterday());
					$end   = $begin;
					break;
				case 'thisweek':
					return (date::getThisWeek());
					break;
				case 'lastweek':
					return (date::getLastWeek());
					break;
				case 'nextmonth':
					return (kuntai::getNextMonth());
					break;
				case 'thismonth':
					return (date::getThisMonth());
					break;
				case 'lastmonth':
					return (date::getLastMonth());
					break;

				case 'thisseason':
					return (date::getThisSeason());
					break;
				case 'thisyear':
					return (date::getThisYear());
					break;
				case 'lastyear':
					return (date::getLastYear());
					break;
				case 'future':
					$begin = kuntai::formatTime(date::tomorrow());
					$end   = '2109-01-01'; //end furture
					break;
				case 'all':
					$begin = '1970-01-01';
					$end   = '2109-01-01'; //end furture
					break;
				case 'before':
					$begin = '1970-01-01';
					$end   = kuntai::formatTime(date::yesterday());
					break;
				default:
					$begin = kuntai::formatTime($period);
					$end   = $begin;
			}
			if (10 == strlen($end)) $end = $end . ' 23:59:59';
		}
		return array('begin' => $begin, 'end' => $end);
	}

	/**
	 * The get year month
	 *
	 * @access public
	 * @param  string $period
	 * @return array of year month.
	 */
	public static function getFirstYearMonth($period)
	{
		$arrayK = kuntai::getBeginEndTime($period);
		$begin  = $arrayK['begin'];
		$year   = substr($begin, 0, 4);
		$month  = substr($begin, 5, 2);
		return array('year' => $year, 'month' => $month);
	}

	/**
	 * The getMonth.
	 *
	 * @access public
	 * @param  date $now
	 * @param  int  $delta
	 * @return int of month[1 to 12].
	 */
	public static function getMonth($delta, $now = 'time()')
	{
		$delta = (int)$delta;
		if ($now == 'time()') {
			$now = time();
		}
		$m = (int)date('m', $now);
		$m = $m + $delta;
		$m = $m % 12;
		if ($m <= 0) {
			$m += 12;
		}
		return $m;
	}

	/**
	 * The strtotime.
	 *
	 * @access public
	 * @param  string $time
	 * @return new date.
	 */
	public static function getMonthBeginEnd($time)
	{
		$newTime  = strtotime($time);
		$begin    = date('Y-m-01 00:00:00', $newTime); //to first day
		$tempTime = strtotime($begin);
		$end      = date('Y-m-d', strtotime("+1 month -1 day", $tempTime)) . ' 23:59:59'; //to last day
		return array('begin' => $begin, 'end' => $end);
	}

	/**
	 * The get month by Season.
	 *
	 * @access public
	 * @param  int $month
	 * @param  int $season
	 * @return array of month.
	 */
	public static function getMonthBySeason($month = '', $season = '')
	{
		$monthArray = array();
		if ('' == $month && $season == '') {
			return $monthArray;
		}
		if ('' == $season) {
			$monthArray[] = $month;
			return $monthArray;
		}
		for ($i = 1; $i < 4; $i++) {
			$currentMonth = sprintf("%02d", 3 * ($season - 1) + $i);
			$monthArray[] = $currentMonth;
		}
		return $monthArray;
	}

	/**
	 * Get begin and end time of this month.
	 *
	 * @access public
	 * @return array
	 */
	public static function getNextMonth()
	{
		$begin = date('Y-m', strtotime('next month')) . '-01 00:00:00';
		$year  = substr($begin, 0, 4);
		$month = substr($begin, 5, 2);
		$days  = cal_days_in_month(CAL_GREGORIAN, $month, $year);
		$end   = $year . '-' . $month . '-' . $days . ' 23:59:59';
		return array('begin' => $begin, 'end' => $end);
	}

	/**
	 * The get month list.
	 *
	 * @access public
	 * @param  int $month
	 * @param  int $season
	 * @return array of month.
	 */
	public static function getMonthList($begin, $end, &$monthArray)
	{
		$monthArray = array();
		$month      = date('Y-m', strtotime($begin)); //to last day
		$monthEnd   = date('Y-m', strtotime($end)); //to last day
		if ($monthEnd < $month) return; //error

		$monthArray[$month] = $month;
		if ($monthEnd == $month) return; //get one
		while (true) {
			$month = date('Y-m', strtotime("$month +1 month")); //to next Month

			if ($month > $monthEnd) return;
			$monthArray[$month] = $month;
		}
	}

	/* show hours like 3.5 for 210 minutes */

	public static function showHours($iMinutes, $i = 1)
	{
		$workhour = ((float)$iMinutes / 60);
		$workhour = number_format($workhour, $i);
		return $workhour;
	}

	/* show hours like 3:30 for 210 minutes */

	public static function showWorkHours($iMinutes)
	{
		if ($iMinutes > 60) {
			if (0 == (int)($iMinutes % 60)) return ((int)($iMinutes / 60)) . ':00';
			return ((int)($iMinutes / 60)) . ':' . ((int)($iMinutes % 60));
		} else if ($iMinutes == 60) {
			return ((int)$iMinutes / 60) . ':00';
		}
		return '0:' . ((int)$iMinutes % 60);
	}

	/**
	 * The strtotime.
	 *
	 * @access public
	 * @param  string $time
	 * @param  date   $now
	 * @return new date.
	 */
	public static function strtotimeMonth($time, $now = 'time()')
	{
		if ($now == 'time()') {
			$now = time();
		}
		$newTime       = strtotime($time, $now);
		$firstday      = date('Y-m-01', $now); //to first day
		$newFirstday   = strtotime("$firstday " . $time);
		$newFirstday_s = date('Y-m-d', $newFirstday);
		$newLastday_s  = date('Y-m-d', strtotime("$newFirstday_s +1 month -1 day")); //to last day
		$newLastday    = strtotime($newLastday_s);

		if ($newTime > $newLastday) {
			return $newLastday;
		} elseif ($newTime < $newFirstday) {
			return $newFirstday;
		}
		return $newTime;
	}

	//---------------------------------
	// 从kuntai base转移过来的

	/**
	 * Append order by.
	 * 
	 * @param  string $orderBy 
	 * @param  string $append 
	 * @access public
	 * @return string
	 */
	public static function appendOrder($orderBy, $append = 'id')
	{
		list($firstOrder) = explode(',', $orderBy);
		$sort = strpos($firstOrder, '_') === false ? '_asc' : strstr($firstOrder, '_');
		return strpos($orderBy, $append) === false ? $orderBy . ',' . $append . $sort : $orderBy;
	}

	public function getCashCodeByProject($project)
	{
		$todoObj = $this->dao->select('cashCode')
			->from(TABLE_PROJECT)
			->where('id')->eq($project)
			->fetch();
		if (!$todoObj) return '';
		return $todoObj->cashCode;
	}
	public function getDeptByAccount($account)
	{
		//获得员工所在的科室id,此处科室可以为股
		$deptId = $this->getDeptIdByAccount($account);
		//获得科室名称
		$deptName = $this->getDeptNameById($deptId);
		//获得科,或者股, 或者部门
		$keywords = mb_substr($deptName, -1, 1, "UTF-8");
		if ('科' == $keywords) return $deptName;
		else if ('部' == $keywords) return $deptName;
		else  return $this->getParentDeptNameById($deptId);
	}
	public function getDeptEmployeesAccount($deptId, $isShowDeleted = false)
	{
		$employees		 = array();
		$employees['']	 = '';

		$deptEmployees = $this->dao->select('a.account')->from(TABLE_USER)->alias('a')
			->leftJoin(TABLE_DEPT)->alias('b')
			->on('a.dept=b.id')
			->where('b.path')->like('%,' . $deptId . ',%')
			->beginIF(!$isShowDeleted)->andWhere('a.deleted')->eq(0)->fi()
			->beginIF(!$isShowDeleted)->andWhere('a.locked')->lt("2030-01-01 00:00:00")->fi()
			->fetchAll('account');
		foreach ($deptEmployees as $key => $employee) {
			$employees[] = $employee->account;
		}

		if (count($employees) == 1) return array();
		return $employees;
	}

	public function getDeptFromDB()
	{
		return $this->dao->select('*')
			->from(TABLE_DEPT)
			->fetchAll('name');
	}
	public function getDeptIdByAccount($account)
	{
		$todoObj = $this->dao->select('dept')
			->from(TABLE_USER)
			->where('account')->eq($account)
			->fetch();
		if (!$todoObj) return '';
		return $todoObj->dept;
	}
	public function getDeptNameByAccount($account)
	{
		$deptId = $this->getDeptIdByAccount($account);
		return $this->getDeptNameById($deptId);
	}

	public function getDeptNameById($id)
	{
		$todoObj = $this->dao->select('name')
			->from(TABLE_DEPT)
			->where('id')->eq($id)
			->fetch();
		if (!$todoObj) return '';
		return $todoObj->name;
	}

	/**
	 * Get simple dept id name pair with path display..
	 * 
	 * @param  int    $rootDeptID 
	 * @param  int    $simple 0,全名称，1： name path，其它：  name[id]
	 * @access public
	 * @return array
	 */
	public function getDeptOptionMenu($rootDeptID = 0, $simple = 1)
	{
		$this->loadModel('dept');
		$deptMenu	 = array();

		//简化模式，显示最后一个名字和路径
		if ($simple) {
			//1的格式： name path，其他的格式：  name[id]
			$str1 = ($simple == 1) ? 'id ,concat(`name`," ",`path`) as nameSimple' : 'id ,concat(`name`," [",`id`,"]") as nameSimple';
			$depts = $this->dao->select($str1)->from(TABLE_DEPT)
				->beginIF($rootDeptID > 0)->where('path')->like('%,' . $rootDeptID . ',%')->fi()
				->orderBy('`path`') //grade asc, 
				->fetchPairs();
			return $depts;
		}

		//全名称模式
		$depts = $this->dao->select('*')->from(TABLE_DEPT)
			->beginIF($rootDeptID > 0)->where('path')->like('%,' . $rootDeptID . ',%')->fi()
			->orderBy('`path`') //grade asc, 
			->fetchAll('id');

		foreach ($depts as $dept) {
			//去除首尾逗号后进行拆分
			$parentDepts = explode(',', trim($dept->path, ','));
			$len		 = count($parentDepts);
			$deptName	 = ''; //"({$dept->grade})";
			for ($i = 0; $i < $len; $i++) {
				$deptName .= '/' . $depts[$parentDepts[$i]]->name;
			}
			$deptName .= "[$dept->id]";
			$deptMenu[$dept->id] =  $deptName;
		}

		return $deptMenu;
	}

	public function getDispatchAccountsByDept($deptID, $isShowDeleted = false)
	{
		$accountArray	 = $this->getDeptEmployeesAccount($deptID, $isShowDeleted);
		$accountObjs = $this->dao->select('account,deptdispatch')->from(TABLE_USER)
			->where('account')->in($accountArray)
			->beginIF(!$isShowDeleted)->andWhere('deleted')->eq(0)->fi()
			->fetchGroup('deptdispatch', 'account');

		return $accountObjs;
	}

	public function getEmployeesByDept($dept, $isShowDeleted = false)
	{
		return $this->dao->select('account,realname,code,worktype')
			->from(TABLE_USER)
			->where('dept')->eq($dept)
			->beginIF(!$isShowDeleted)->andWhere('deleted')->eq(0)->fi()
			->fetchAll();
	}
	/**
	 * get module file contents
	 * 
	 * @moduleName  module name
	 * @refpath  file path ref to module base folder
	 * @access public
	 * @return string of file contents
	 */
	public function getModuleFileContents($moduleName, $refpath)
	{
		$moduleName  = strtolower(trim($moduleName));
		$modulePath  = $this->app->getModulePath($moduleName);

		$contents = '';
		$mainJsFile   = $modulePath . $refpath;
		if (file_exists($mainJsFile))   $contents .= file_get_contents($mainJsFile);
		return $contents;
	}

	/**
	 * getMonthList .
	 * @param  bool  $withblank 
	 * @access public
	 * @return void
	 */
	public static function getMonthListOneYear($withblank = 0, $fill0 = true)
	{
		$List = array();
		if ($withblank) $List[''] = ''; //blank
		$prefix = ($fill0) ? '0' : '';
		for ($i = 1; $i < 10; $i++) {
			$month = $prefix . $i;
			$List[$month] = $month;
		}
		for ($i = 10; $i <= 12; $i++) {
			$List[$i] = $i;
		}
		return $List;
	}

	public function getParentDeptIdById($id)
	{
		$currentDept = $this->dao->select('parent')
			->from(TABLE_DEPT)
			->where('id')->eq($id)
			->fetch();
		if (!$currentDept) return 0;
		$parentDeptId = $currentDept->parent;
		if (0 == $parentDeptId) return 0;
		$deptParent = $this->dao->select('id')
			->from(TABLE_DEPT)
			->where('id')->eq($parentDeptId)
			->fetch();
		if (!$deptParent) return 0;
		return $deptParent->id;
	}
	public function getParentDeptNameById($id)
	{
		$currentDept = $this->dao->select('*')
			->from(TABLE_DEPT)
			->where('id')->eq($id)
			->fetch();
		if (!$currentDept) return '';
		$parentDeptId = $currentDept->parent;
		if (0 == $parentDeptId) return '';
		$deptParent = $this->dao->select('name')
			->from(TABLE_DEPT)
			->where('id')->eq($parentDeptId)
			->fetch();
		if (!$deptParent) return '';
		return $deptParent->name;
	}
	public function getProjectNameByProject($project)
	{
		if ($project == '') return '';
		$todoObj = $this->dao->select('name')
			->from(TABLE_PROJECT)
			->where('id')->eq($project)
			->fetch();
		if (!$todoObj) return '';
		return $todoObj->name;
	}
	public function getProjectNameByProjectPairs($project)
	{
		$projectObj = $this->dao->select('name')
			->from(TABLE_PROJECT)
			->where('id')->eq($project)
			->fetch();
		$ret = $projectObj->name;
		if ('' == $ret) $ret = '项目代号不存在';
		$projectName = array();
		$projectName[] = $ret;
		return $projectName;
	}
	public function getProjectsByProductID($productID)
	{
		$projects = array(); //产品下的所有项目组成的数组
		$projectArray = $this->dao->select('*')->from(TABLE_PROJECTPRODUCT)
			->where('product')->eq($productID)
			->fetchAll();
		foreach ($projectArray as $project) {
			$projects[] = $project->project;
		}
		return $projects;
	}
	/**
	 * get Real name By Account
	 * 
	 * @account  account string
	 * @access public
	 * @return string of real name
	 */
	public function getRealnameByAccount($account)
	{
		$rs = $this->dao->select('realname')
			->from(TABLE_USER)
			->where('account')->eq($account)
			->fetch();
		if (!$rs) return $account;
		return $rs->realname;
	}
	public function getSectionIdByDeptID($deptID)
	{
		$pathObj = $this->dao->select('path')->from(TABLE_DEPT)
			->where('id')->eq($deptID)
			->fetch();
		if (!$pathObj) {
			return 0;
		}
		$path = $pathObj->path;
		$pathArray = explode(',', $path);
		return $pathArray[2];
	}
	public function getSectionNameByAccount($account)
	{
		//获得员工所在的科室id,此处科室可以为股
		$deptId = $this->getDeptIdByAccount($account);
		//获得科室名称
		$deptName = $this->getDeptNameById($deptId);
		//获得科,或者股, 或者部门
		$keywords = mb_substr($deptName, -1, 1, "UTF-8");
		if ('科' == $keywords) return $this->getParentDeptNameById($deptId);
		else if ('部' == $keywords) return $deptName;
		else {
			while ('部' != $keywords) {
				$deptId = $this->getParentDeptIdById($deptId);
				//获得科室名称
				$deptName = $this->getDeptNameById($deptId);
				//获得科,或者股, 或者部门
				$keywords = mb_substr($deptName, -1, 1, "UTF-8");
			}
			return $deptName;
		}
	}
	public function getSectionNameByDeptID($deptID)
	{
		$pathObj = $this->dao->select('path')->from(TABLE_DEPT)
			->where('id')->eq($deptID)
			->fetch();
		if (!$pathObj) {
			return '';
		}
		$path = $pathObj->path;
		$pathArray = explode(',', $path);
		return $this->getDeptNameById($pathArray[2]);
	}
	public function getUserCodeByAccount($account)
	{
		$todoObj = $this->dao->select('code')
			->from(TABLE_USER)
			->where('account')->eq($account)
			->fetch();
		if (!$todoObj) return '';
		return $todoObj->code;
	}
	public function getUserInfoByAccount($account)
	{
		$userObj = $this->dao->select('*')
			->from(TABLE_USER)
			->where('account')->eq($account)
			->fetch();
		return $userObj;
	}
	public function getWorkMinutes($workhours, $begin = '', $end = '')
	{
		if ($workhours == '') {
			$endArray = explode(':', $end);
			$beginTime = 60 * ((int)substr($begin, 0, 2)) + ((int)substr($begin, 2, 2));
			$endTime = 60 * ((int)substr($end, 0, 2)) + ((int)substr($end, 2, 2));
			if ($begin > $end) return $beginTime + 60 * 12 - $endTime;
			else return $endTime - $beginTime;
		}
		$array = explode(':', $workhours);
		return 60 * $array[0] + $array[1];
	}
	//通过用户名获得员工类型，默认为0
	public function getWorkTypeByAccount($account)
	{
		$worktype = $this->dao->select('worktype')
			->from(TABLE_USER)
			->where('account')->eq($account)
			->fetch();
		if (!$worktype) return 0;
		return $worktype->worktype;
	}

	/**
	 * getYearList .
	 * @param  int  $year 
	 * @access public
	 * @return void
	 */
	public static function getYearList($year = 10)
	{
		$yearList = array();
		for ($i = -1; $i < $year; $i++) {
			$yearList[date('Y') - $i] = (date('Y') - $i);
		}
		return $yearList;
	}


	/**
	 * Print the main menu.
	 * 
	 * @param  string $moduleName 
	 * @static
	 * @access public
	 * @return void
	 */
	public static function printMainmenu($moduleName, $ActiveModuleName = '')
	{
		global $app, $lang;

		/* Set the main main menu. */
		$mainMenu = ($ActiveModuleName) ? $ActiveModuleName : $app->getModuleName();;

		/* Print all main menus. */
		$menu       = customModel::getModuleMenu($moduleName);
		$activeName = 'active';

		echo "<ul class='nav'>\n";


		echo "<li data-id='myhome'>" . html::a(helper::createLink('my', 'index'), '<i class="icon-home"></i>') . "</li>\n";
		foreach ($menu as $menuItem) {
			if (isset($menuItem->hidden) && $menuItem->hidden) continue;
			$active = $menuItem->name == $mainMenu ? "class='$activeName'" : '';
			$link   = commonModel::createMenuLink($menuItem);
			echo "<li $active data-id='$menuItem->name'><a href='$link' $active>$menuItem->text</a></li>\n";
		}

		$isGuest = $app->user->account == 'guest';
		$logurl = '';
		if ($isGuest) {
			$logurl = html::a(helper::createLink('user', 'login'), $lang->login);
		} else {
			$logurl = html::a(helper::createLink('user', 'logout'), $lang->logout);
		}
		echo "<li data-id='myhome' class = 'pull-right'>" . $logurl . "</li>\n";
		echo "<li data-id='myhome' class = 'pull-right'>" . html::a('#', "<i class='icon-user'></i> " . $app->user->realname) . "</li>\n";

		echo "</ul>\n";
	}


	//Replace %s in String 
	static public function Replace(&$error, &$replace)
	{
		$pos	 = strpos($error, '%s');
		if ($pos === false) return;
		$error	 = substr($error, 0, $pos) . $replace . substr($error, $pos + 2);
	}

	public function writeDataToFile($data)
	{
		//如zentao在D盘,则路径为D:\xampp\zentao\tmp\log\log.txt
		$myfile = fopen("../tmp/log/log.txt", "a") or die("Unable to open file!");
		if ('' == $data) $data = 'Data is empty!';
		$data  = date('Y-m-d h:i:s') . '  =>  ' . $data  . "\r\n";
		fwrite($myfile, $data);
		fclose($myfile);
	}
}
